// Classes for feature and feature vector

#ifndef FEATURE_VEC_HPP
#define FEATURE_VEC_HPP

#include <list>
#include "MLConfig.hpp"

//////////////////////////////////////////////////////////////////////

// Feature = feature name + feature value

template <typename FeatureName_>
class BinaryFeature {
public:
  typedef FeatureName_ FeatureName;
  typedef int FeatureValue;
private:
  FeatureName n;
public:
  BinaryFeature() {}
  explicit BinaryFeature(FeatureName n_, FeatureValue = 1) : n(n_) {}
  ~BinaryFeature() {}

  FeatureName name() const { return n; }
  FeatureValue value() const { return 1; }
};

template <typename FeatureName_, typename Number_>
class RealFeature {
public:
  typedef FeatureName_ FeatureName;
  typedef Number_ FeatureValue;
private:
  FeatureName n;
  FeatureValue v;
public:
  RealFeature() {}
  explicit RealFeature(FeatureName n_, FeatureValue v_ = 1.0) : n(n_), v(v_) {}
  ~RealFeature() {}

  FeatureName name() const { return n; }
  FeatureValue value() const { return v; }
};

//////////////////////////////////////////////////////////////////////

// Feature vector = list of features

template <typename Feature_, typename Number_> //, typename HashFunc, typename HashEq>
class FeatureVec : public std::list<Feature_> {
public:
  typedef Feature_ Feature;
  typedef typename Feature::FeatureName FeatureName;
  typedef typename Feature::FeatureValue FeatureValue;

  //Number product(const FeatureMap<FeatureName>& fmap) 
  //Number product(const WeightHash<FeatureName,HashFunc,HashEq>& fmap) {
  template <typename WeightMap>
  Number_ product(const WeightMap& fmap) {
    Number_ x = 0.0;
    for(typename std::list<Feature>::const_iterator it = this->begin(); it != this->end(); ++it) {
      x += fmap.weight(it->name()) * it->value();
    }
    return x;
  }
};

template <typename FeatureName_, typename Number_> //, typename HashFunc, typename HashEq>
class FeatureVec<BinaryFeature<FeatureName_>, Number_ > : public std::list<BinaryFeature<FeatureName_> > {
public:
  typedef BinaryFeature<FeatureName_> Feature;
  typedef typename Feature::FeatureName FeatureName;
  typedef typename Feature::FeatureValue FeatureValue;

  //Number product(const FeatureMap<FeatureName>& fmap) 
  //Number product(const WeightHash<FeatureName,HashFunc,HashEq>& fmap) {
  template <typename WeightMap>
  Number_ product(const WeightMap& fmap) {
    Number_ x = 0.0;
    for(typename std::list<Feature>::const_iterator it = this->begin(); it != this->end(); ++it) {
      x += fmap.weight(it->name());
      //cerr << "weight " << fmap.weight(it->name());
    }
    return x;
  }
};

#endif // FEATURE_VEC_HPP

