// portability for hash_map

#ifndef HASH_MAP_HPP
#define HASH_MAP_HPP

#include "MLConfig.hpp"

#if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3)
  #define HAVE_TR1_UNORDERED_MAP
#elif __GNUC__ == 3 && __GNUC_MINOR__ == 0 // hack for gcc 3.0.x
  #define HAVE_EXT_HASH_MAP_GNU_3_0
#elif (__GNUC__ == 4 && __GNUC_MINOR__ <= 2) || (__GNUC__ == 3)
  #define HAVE_EXT_HASH_MAP_GNU
#elif defined(__MSVC_VER__) || defined(__sgi__) || (__GNUC__ == 2 && __GNUC_MINOR__ >= 95)
  #define HAVE_HASH_MAP
#else
  // assuming <unordered_map>
  #define HAVE_UNORDERED_MAP
#endif

#if defined(HAVE_UNORDERED_MAP)

#include <unordered_map>
template <typename Key>
class HashFun : public std::hash<Key> {};
template <typename Key, typename Value, typename Fun = std::hash<Key>, typename Pred = std::equal_to<Key> >
class HashMap : public std::unordered_map<Key, Value, Fun, Pred> {};

#elif defined(HAVE_TR1_UNORDERED_MAP)

#include <tr1/unordered_map>
template <typename Key>
class HashFun : public std::tr1::hash<Key> {};
template <typename Key, typename Value, typename Fun = std::tr1::hash<Key>, typename Pred = std::equal_to<Key> >
class HashMap : public std::tr1::unordered_map<Key, Value, Fun, Pred> {};

#elif defined(HAVE_EXT_HASH_MAP_GNU_3_0)

#include <ext/hash_map>
template <typename Key>
class HashFun : public std::hash<Key> {};
template <typename Key, typename Value, typename Fun = std::hash<Key>, typename Pred = std::equal_to<Key> >
class HashMap : public __gnu_cxx::hash_map<Key, Value, Fun, Pred> {};

#elif defined (HAVE_EXT_HASH_MAP_GNU)

#include <ext/hash_map>
template <typename Key>
class HashFun : public __gnu_cxx::hash<Key> {};
template <typename Key, typename Value, typename Fun = __gnu_cxx::hash<Key>, typename Pred = std::equal_to<Key> >
class HashMap : public __gnu_cxx::hash_map<Key, Value, Fun, Pred> {};

#elif defined(HAVE_EXT_HASH_MAP)

#include <ext/hash_map>
template <typename Key>
class HashFun : public ext::hash<Key> {};
template <typename Key, typename Value, typename Fun = ext::hash<Key>, typename Pred = std::equal_to<Key> >
class HashMap : public ext::hash_map<Key, Value, Fun, Pred> {};

#elif defined(HAVE_HASH_MAP)

#include <hash_map>
template <typename Key>
class HashFun : public std::hash<Key> {};
template <typename Key, typename Value, typename Fun = std::hash<Key>, typename Pred = std::equal_to<Key> >
class HashMap : public std::hash_map<Key, Value, Fun, Pred> {};

#else

#include <hash_map.h>
template <typename Key>
class HashFun : public std::hash<Key> {};
template <typename Key, typename Value, typename Fun = std::hash<Key>, typename Pred = std::equal_to<Key> >
class HashMap : public hash_map<Key, Value, Fun, Pred> {};

#endif

class StringHashFun {
public:
  size_t operator()(const std::string& s) const {
    size_t x = 0;
    for (std::string::const_iterator it = s.begin();
         it != s.end();
         ++it) {
      x = x * 31 + *it;
    }
    return x;
  }
};

template <typename Value, typename Fun = StringHashFun, typename Pred = std::equal_to<std::string> >
class StringHashMap : public HashMap<std::string, Value, Fun, Pred> {};

#endif // HASH_MAP_HPP

