#ifndef STACK_HPP
#define STACK_HPP

#include <vector>
#include <string>
#include <list>

typedef int WordID;  // position of the word in a sentence
typedef int LabelID;

typedef std::vector<std::string> Word;

class Dependency {
public:
  WordID head;
  std::string label;
  Dependency(WordID h, const std::string& l) : head(h), label(l) {}
  Dependency() : head(0) {}
};

enum Action {
  SHIFT = 0,
  REDUCE_LEFT = 1,
  REDUCE_RIGHT = 2
};

class StackElem {
public:
  WordID word_id;
  std::list<StackElem> left;   // IDs of left children
  std::list<StackElem> right;  // IDs of right children
  StackElem(WordID w)
    : word_id(w) {}
};

class Stack {
private:
  std::vector<StackElem> stack; // stack of constituents
  //std::list<LabelID> actions;  // list of previous actions

public:
  Stack() {}
  ~Stack() {}

public:
  // stack operations
  size_t size() const { return stack.size(); }
  void clear() {
    stack.clear();
    //actions.clear();
  }
  const StackElem& operator[](size_t i) const { return stack[i]; }
  typedef std::vector<StackElem>::const_iterator const_iterator;
  const_iterator begin() const { return stack.begin(); }
  const_iterator end() const { return stack.end(); }

  const StackElem& leftElem() const { return stack[stack.size() - 2]; }
  const StackElem& rightElem() const { return stack[stack.size() - 1]; }
  StackElem& leftElem() { return stack[stack.size() - 2]; }
  StackElem& rightElem() { return stack[stack.size() - 1]; }

  //const std::list<LabelID>& prevActions() const { return actions; }

  void reduceRight() {
    rightElem().left.push_front(leftElem());
    leftElem() = rightElem();
    stack.pop_back();
    //actions.push_back(REDUCE_RIGHT);
    return;
  }
  void reduceLeft() {
    leftElem().right.push_back(rightElem());
    stack.pop_back();
    //actions.push_back(REDUCE_LEFT);
    return;
  }
  void shift(WordID word_id) {
    stack.push_back(StackElem(word_id));
    //actions.push_back(SHIFT);
  }

  void rewindReduceRight() {
    stack.push_back(rightElem());
    leftElem() = rightElem().left.front();
    rightElem().left.pop_front();
  }
  void rewindReduceLeft() {
    stack.push_back(rightElem().right.back());
    leftElem().right.pop_back();
  }
  void rewindShift() {
    stack.pop_back();
  }
};

#endif // STACK_HPP
