// string <-> int indexer

#ifndef STRINGINDEXER
#define STRINGINDEXER

#include "Exception.hpp"
#include "HashMap.hpp"
#include <map>
#include <vector>
#include <string>
#include <sstream>
#include <iostream>
#include <iterator>

class StringIndexer {
  private:
    StringHashMap<int> indexer;
    std::vector<std::string> reverse_indexer;
    bool add_new_token_if_not_exist;

  public:
    StringIndexer(bool a = false) : add_new_token_if_not_exist(a) {}
    virtual ~StringIndexer() {}

  public:
    int getint(const std::string &token) const {
      StringHashMap<int>::const_iterator ret = indexer.find(token);
      //if(ret==indexer.end()) throw Exception("const int& StringIndexer::getint(const std::string &token) const: text label ,," + token + "'' not found in this indexer of labels");
      if (ret == indexer.end()) return -1;
      return ret->second;
    }

    int getint(const std::string& token, bool add_new_token) {
      StringHashMap<int>::const_iterator ret = indexer.find(token);
      if (ret == indexer.end()) {
        if (add_new_token) {
          int new_id = reverse_indexer.size();
          indexer[token] = new_id;
          reverse_indexer.push_back(token);
          return new_id;
        } else {
          return -1;
        }
      }
      return ret->second;
    }

    int mapint(const std::string& token) {
      return mapint(token, add_new_token_if_not_exist);
    }

    int mapint(const std::string& token, bool add_new_token) {
      return getint(token, add_new_token);
    }
    
    bool hasint(const std::string &token) const {
      if(indexer.find(token)==indexer.end()) return false;
      else return true;
    }

    const std::string &getString(int label) const {
      if((((size_t)label)>=reverse_indexer.size())){
        std::ostringstream out;
        out << "const string& StringIndexer::getString(const int &label) const: label " << label << " not found in this indexer of labels" << std::ends;
        int * bug = NULL;
        out << *bug;
        throw Exception(out.str());
      }
      return reverse_indexer[label];
    }
    
    int addToken(const std::string &token) {
      StringHashMap<int>::value_type p(token,reverse_indexer.size());
      std::pair<StringHashMap<int>::iterator,bool> ret=indexer.insert(p);
      if(ret.second) reverse_indexer.push_back(token);
      return ret.first->second;
    }

    int size() const {
      return indexer.size();
    }
    
    void read(std::istream &input) {
      std::string token;
      while(input >> token){
        addToken(token);
      }
    }

    void print(std::ostream &out) const {
        //copy(reverse_indexer.begin(), reverse_indexer.end(), std::ostream_iterator<std::string>(out, "\n"));
      for(std::vector<std::string>::const_iterator iter=reverse_indexer.begin();
          iter!=reverse_indexer.end(); ++iter) {
        out << (*iter) << std::endl;
      }
    }
};


#endif
