#ifndef TIMER_HPP
#define TIMER_HPP

#include <sys/time.h>
#include <iostream>
#include <sstream>

class Timer {
private:
  struct timeval begin, end;
  size_t real_usec;
  int num_intervals;

public:
  Timer() {
    begin.tv_sec = 0;
    begin.tv_usec = 0;
    end.tv_sec = 0;
    end.tv_usec = 0;
    real_usec = 0;
    num_intervals = 0;
  }
  virtual ~Timer() {}

  void start() {
    gettimeofday(&begin, NULL);
  }
  void stop() {
    gettimeofday(&end, NULL);
    real_usec += (end.tv_sec - begin.tv_sec) * 1000000;
    real_usec += end.tv_usec - begin.tv_usec;
    ++num_intervals;
  }

  void clear() {
    real_usec = 0;
    num_intervals = 0;
  }

  std::string printInterval() const {
    std::ostringstream os;
    if (real_usec >= 60000000) {
      os << (int)(real_usec / 60000000) << " min ";
    }
    if (real_usec % 60000000 >= 1000000) {
      os << ((real_usec % 60000000) / 1000000) << " sec ";
    }
    os << (real_usec % 1000000) << " microsec";
    return os.str();
  }

  int numIntervals() const { return num_intervals; }
  size_t realUsec() const { return real_usec; }
};

#endif // TIMER_HPP
