// WeightMap: feature name -> feature weight

#ifndef WEIGHTMAP_HPP
#define WEIGHTMAP_HPP

#include <map>
#include <vector>
//#include <hash_map.h>
#include "MLConfig.hpp"
#include "HashMap.hpp"

// hash version
template <typename FeatureName_, typename Number_, typename HashFunc_, typename Equals_>
class WeightHash : public HashMap<FeatureName_,Number_,HashFunc_,Equals_> {
public:
  typedef FeatureName_ FeatureName;
  typedef Number_ Number;

  WeightHash(size_t) {}

  Number weight(const FeatureName& f) const {
    typename HashMap<FeatureName_,Number_,HashFunc_,Equals_>::const_iterator f_it
                         = this->find(f);
    if(f_it == this->end()) return 0.0;
    return f_it->second;
  }
  void write(std::ostream &out) {
      for(typename HashMap<FeatureName_,Number_,HashFunc_,Equals_>::const_iterator iter=this->begin();
          iter!=this->end(); ++iter) {
        if (iter->second != 0.0) {
          out << iter->first << "\t" << iter->second << std::endl;
        }
      }
  }
  void read(std::istream &in) {
      while(true) {
        FeatureName f;
        //this->readFeature(in, f);
        in >> f;
        if (!in) break;
        Number value = 0;
        in >> value;
        if (!in) break;
        if (value != 0.0) {
          HashMap<FeatureName_,Number_,HashFunc_,Equals_>::operator[](f) = value;
        }
      }
  }
};

// map version
// template <typename FeatureName_, typename Number_, typename Compare_ = std::less<FeatureName_> > 
// class WeightMap : public std::map<FeatureName_, Number_, Compare_> {
// public:
//   typedef FeatureName_ FeatureName;
//   typedef Number_ Number;

//   WeightMap(size_t) {}

//   Number weight(const FeatureName& f) const {
//     typename std::map<FeatureName, Number_, Compare_>::const_iterator f_it = this->find(f);
//     if(f_it == this->end()) return 0.0;
//     return f_it->second;
//   };
// };

// vector version
template <typename Number_>
class WeightVector : public std::vector<Number_> {
public:
  typedef typename std::vector<Number_>::size_type FeatureName;
  typedef Number_ Number;

  // iterator emulates map-like access
public:
  class index {
  private:
    index& operator=(const index&) {}
  public:
    size_t first;
    Number& second;
    index(size_t f, Number& s) : first(f), second(s) {}
    index(const index& x) : first(x.first), second(x.second) {}
    index* operator->() {
      return this;
    }
  };
  class const_index {
  private:
    const_index& operator=(const const_index&) {}
  public:
    size_t first;
    const Number& second;
    const_index(size_t f, const Number& s) : first(f), second(s) {}
    const_index(const const_index& x) : first(x.first), second(x.second) {}
    const_index* operator->() {
      return this;
    }
  };

  class iterator : public std::iterator<std::random_access_iterator_tag, index> {
  public:
    typename std::vector<Number>::iterator begin;
    typename std::vector<Number>::iterator iter;

  public:
    iterator(typename std::vector<Number>::iterator b,
             typename std::vector<Number>::iterator i)
      : begin(b), iter(i) {}
    iterator(const iterator& i)
      : begin(i.begin), iter(i.iter) {}
    iterator& operator=(const iterator& i) {
      begin = i.begin;
      iter = i.iter;
    }

    index operator*() {
      return index(iter - begin, *iter);
    }
    index operator[](ptrdiff_t a) {
      return index(iter - begin + a, *(iter + a));
    }
    index operator->() {
      return index(iter - begin, *iter);
    }

    iterator& operator++() {
      ++iter;
      return *this;
    }
    iterator operator++(int) {
      iterator it(*this);
      ++iter;
      return it;
    }
    iterator& operator+=(ptrdiff_t a) {
      iter += a;
      return *this;
    }
    iterator& operator--() {
      --iter;
      return *this;
    }
    iterator operator--(int) {
      iterator it(*this);
      --iter;
      return it;
    }
    iterator& operator-=(ptrdiff_t a) {
      iter -= a;
      return *this;
    }
    bool operator==(iterator i) const {
      return begin == i.begin && iter == i.iter;
    }
    bool operator!=(iterator i) const {
      return begin != i.begin || iter != i.iter;
    }
    bool operator<(iterator i) const {
      return iter < i.iter;
    }
    bool operator>(iterator i) const {
      return iter > i.iter;
    }
    iterator operator+(ptrdiff_t a) const {
      return iterator(begin, iter + a);
    }
    iterator operator-(ptrdiff_t a) const {
      return iterator(begin, iter - a);
    }
    ptrdiff_t operator-(const iterator i) const {
      return i.iter - iter;
    }
  };

  class const_iterator : public std::iterator<std::random_access_iterator_tag, const_index> {
  public:
    typename std::vector<Number>::const_iterator begin;
    typename std::vector<Number>::const_iterator iter;

  public:
    const_iterator(typename std::vector<Number>::const_iterator b,
                   typename std::vector<Number>::const_iterator i)
      : begin(b), iter(i) {}
    const_iterator(const const_iterator& i)
      : begin(i.begin), iter(i.iter) {}
    const_iterator(const iterator& i)
      : begin(i.begin), iter(i.iter) {}
    const_iterator& operator=(const const_iterator& i) {
      begin = i.begin;
      iter = i.iter;
    }

    const_index operator*() {
      return const_index(iter - begin, *iter);
    }
    const_index operator[](ptrdiff_t a) {
      return const_index(iter - begin + a, *(iter + a));
    }
    const_index operator->() {
      return const_index(iter - begin, *iter);
    }

    const_iterator& operator++() {
      ++iter;
      return *this;
    }
    const_iterator operator++(int) {
      const_iterator it(*this);
      ++iter;
      return it;
    }
    const_iterator& operator+=(ptrdiff_t a) {
      iter += a;
      return *this;
    }
    const_iterator& operator--() {
      --iter;
      return *this;
    }
    const_iterator operator--(int) {
      const_iterator it(*this);
      --iter;
      return it;
    }
    const_iterator& operator-=(ptrdiff_t a) {
      iter -= a;
      return *this;
    }
    bool operator==(const_iterator i) const {
      return begin == i.begin && iter == i.iter;
    }
    bool operator!=(const_iterator i) const {
      return begin != i.begin || iter != i.iter;
    }
    bool operator<(const_iterator i) const {
      return iter < i.iter;
    }
    bool operator>(const_iterator i) const {
      return iter > i.iter;
    }
    const_iterator operator+(ptrdiff_t a) const {
      return const_iterator(begin, iter + a);
    }
    const_iterator operator-(ptrdiff_t a) const {
      return const_iterator(begin, iter - a);
    }
    ptrdiff_t operator-(const const_iterator i) const {
      return i.iter - iter;
    }
  };

public:
  WeightVector(size_t n) : std::vector<Number>(n, 0.0) {}

  iterator find(FeatureName f) {
    if (f < this->size()) {
      return begin() + f;
    } else {
      return end();
    }
  }

  iterator begin() {
    return iterator(std::vector<Number>::begin(), std::vector<Number>::begin());
  }
  iterator end() {
    return iterator(std::vector<Number>::begin(), std::vector<Number>::end());
  }
  const_iterator begin() const {
    return const_iterator(std::vector<Number>::begin(), std::vector<Number>::begin());
  }
  const_iterator end() const {
    return const_iterator(std::vector<Number>::begin(), std::vector<Number>::end());
  }

  void clear() {
    assign(this->size(), 0.0);
  }

  Number weight(FeatureName f) const {
    return (*this)[f];
  }
  void write(std::ostream &out) {
      for(typename WeightVector<Number_>::const_iterator iter=this->begin();
          iter!=this->end(); ++iter) {
        if (iter->second != 0.0) {
          out << iter->first << "\t" << iter->second << std::endl;
        }
      }
  }
  void read(std::istream &in) {
      while(true) {
        FeatureName f;
        in >> f;
        if (!in) break;
        Number value = 0;
        in >> value;
        if (!in) break;
        //if(this->size()<=(int)f) cerr << "index " << (int)f << std::endl;
        if (value != 0.0) {
          std::vector<Number_>::operator[](f) = value;
        }
      }
  }
};

// matrix version
template <typename FeatureName_, typename Number_, typename HashFunc_, typename Equals_>
class WeightMatrix : public HashMap<FeatureName_,WeightHash<FeatureName_,Number_,HashFunc_,Equals_>,HashFunc_,Equals_>
{
public:
  typedef FeatureName_ FeatureName;
  typedef Number_ Number;

  Number weight(const FeatureName& f1, const FeatureName& f2) const {
    typename HashMap<FeatureName_,WeightHash<FeatureName_,Number_,HashFunc_,Equals_>,HashFunc_,Equals_>::const_iterator f1_it
                         = this->find(f1);
    if(f1_it == this->end()) return 0.0;
    else {
      typename HashMap<FeatureName_,Number_,HashFunc_,Equals_>::const_iterator f2_it
                         = f1_it->find(f2);
        if(f2_it == f1_it->end()) return 0.0;
        return f2_it->second;
    }
  }
};

#endif // WEIGHTMAP_HPP
